import React, { useState, useEffect, useMemo } from 'react';

const useForm = (initialForm = {}, formValidations = {}) => {
    // State for form values
    const [formState, setFormState] = useState(initialForm);
    // State for form validation messages
    const [formValidation, setFormValidation] = useState({});
    // State for form submission status
    const [isSubmitted, setIsSubmitted] = useState(false);
    const [touchedFields, setTouchedFields] = useState(new Set());
    // Effect to create validators on component mount
    useEffect(() => {
        createValidators(formState);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    // Memoized value to check if form is valid
    const isFormValid = useMemo(() => {
        if (Object.keys(formState).length === 0)
            return false;
        for (const formValue of Object.keys(formValidation)) {
            if (formValidation[formValue] !== null)
                return false;
        }
        return true;
    }, [formValidation, formState]);
    // Handler for input change events
    const onInputChange = (event) => {
        const target = event.target;
        let value;
        if (target instanceof HTMLInputElement) {
            if (target.type === "checkbox") {
                value = target.checked;
            }
            else if (target.type === "file" && target.files) {
                value = target.files[0];
            }
            else {
                // Convertir el valor a un número si es posible
                value = isNaN(Number(target.value)) ? target.value : Number(target.value);
            }
        }
        else {
            // Convertir el valor a un número si es posible
            value = isNaN(Number(target.value)) ? target.value : Number(target.value);
        }
        const name = target.name;
        // Update form state with new value and re-validate
        setFormState((prevState) => {
            const newState = {
                ...prevState,
                [name]: value,
            };
            createValidators(newState);
            return newState;
        });
    };
    // Handler to reset form to initial state
    const onResetForm = () => {
        setFormState(() => {
            const newState = initialForm;
            createValidators(newState);
            return newState;
        });
    };
    // Function to create validators for form fields
    const createValidators = async (state) => {
        const formCheckedValues = {};
        for (const formField of Object.keys(formValidations)) {
            const [fn, errorMessage] = formValidations[formField];
            const value = state[formField];
            const isValid = await Promise.resolve(fn(value));
            formCheckedValues[formField] = isValid ? null : errorMessage;
        }
        // Update form validation state
        setFormValidation(formCheckedValues);
    };
    const onBlur = (event) => {
        const name = event.target.name;
        setTouchedFields((prev) => new Set(prev).add(name));
        createValidators(formState);
    };
    // Handler for form submission
    const handleSubmit = (onSubmit) => {
        return async (event) => {
            event.preventDefault();
            if (isFormValid) {
                setIsSubmitted(true);
                await onSubmit(formState);
                setIsSubmitted(false);
            }
        };
    };
    // Return form state, handlers and validation state
    return {
        ...formState,
        formState,
        setFormState,
        onInputChange,
        onResetForm,
        formValidation,
        ...formValidation,
        isFormValid,
        handleSubmit,
        isSubmitted,
        onBlur,
        touchedFields,
    };
};

const isValidEmail = (value) => {
    if (typeof value !== "string" || !value) {
        return false;
    }
    const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
    return emailRegex.test(value);
};
const min = (minValue) => (value) => {
    if (typeof value === "number") {
        return value >= minValue;
    }
    else if (typeof value === "string") {
        return value.length >= minValue;
    }
    return false;
};
const max = (maxValue) => (value) => {
    if (typeof value === "number") {
        return value <= maxValue;
    }
    else if (typeof value === "string") {
        return value.length <= maxValue;
    }
    return false;
};
const validateStrongPassword = (password) => {
    const strongPasswordRegex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$/;
    return strongPasswordRegex.test(password);
};
const required = (value) => {
    return value !== null && value !== undefined && value !== "";
};
const equalTo = (compareValue) => (value) => {
    return value === compareValue;
};
const isNumber = (value) => {
    return typeof value === "number";
};
const isBoolean = (value) => {
    return typeof value === "boolean";
};
const isString = (value) => {
    return typeof value === "string";
};
const pattern = (regex) => (value) => {
    if (typeof value !== "string") {
        return false;
    }
    return regex.test(value);
};
const optional = (validator) => (value) => {
    if (value === null || value === undefined || value === "") {
        return true;
    }
    return validator(value);
};
const isTrue = (value) => {
    return value === true;
};
const isFalse = (value) => {
    return value === false;
};
const isImage = (value) => {
    console.log(value);
    if (value instanceof File) {
        return value.type.includes("image");
    }
    return false;
};
const minimunQuantity = (minValue) => (value) => {
    if (typeof value === "number") {
        return value >= minValue;
    }
    return false;
};
const maximumQuantity = (maxValue) => (value) => {
    if (typeof value === "number") {
        return value <= maxValue;
    }
    return false;
};
const isValidUrl = (value) => {
    if (typeof value !== "string") {
        return false;
    }
    try {
        new URL(value);
        return true;
    }
    catch (_) {
        return false;
    }
};
const isValidPhoneNumber = (value) => {
    if (typeof value !== "string") {
        return false;
    }
    const phoneNumberRegex = /^\+?[1-9]\d{1,14}$/;
    return phoneNumberRegex.test(value);
};
const isValidUSZipCode = (value) => {
    if (typeof value !== "string") {
        return false;
    }
    const zipCodeRegex = /^\d{5}(-\d{4})?$/;
    return zipCodeRegex.test(value);
};

const FormValidate = ({ errors, children }) => {
    const renderChildren = (children) => {
        return React.Children.map(children, (child) => {
            if (!React.isValidElement(child)) {
                return child;
            }
            if (child.props.name) {
                const fieldName = child.props.name;
                const fieldNameValid = `${fieldName}Valid`;
                const errorMessage = errors[fieldNameValid];
                return (React.createElement("div", null,
                    React.cloneElement(child, child.props),
                    errorMessage && (React.createElement("div", { key: `error-${fieldName}`, style: { color: "red" } }, errorMessage))));
            }
            if (child.props.children) {
                return React.cloneElement(child, {
                    ...child.props,
                    children: renderChildren(child.props.children),
                });
            }
            return child;
        });
    };
    return React.createElement("div", null, renderChildren(children));
};

export { FormValidate, equalTo, isBoolean, isFalse, isImage, isNumber, isString, isTrue, isValidEmail, isValidPhoneNumber, isValidUSZipCode, isValidUrl, max, maximumQuantity, min, minimunQuantity, optional, pattern, required, useForm, validateStrongPassword };
